@@ -45,9 +45,8 @@
 #endif
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
-#ifdef HAVE_V4L2
-  #include <libv4l2.h>
-  #include <linux/videodev2.h>
+#ifdef HAVE_CAM
+  #include "../libcamera/camera.h"
 #endif
 #include "userlist.h"
 #include "media.h"
@@ -206,11 +205,21 @@ gboolean handledata(GIOChannel* iochannel, GIOCondition condition, gpointer data
   }
   if(!strncmp(buf, "Connection ID: ", 15)) // Our initial nickname is "guest-" plus our connection ID
   {
+    write(tc_client_in[1], "/color\n", 7); // Check which random color tc_client picked
     unsigned int length=strlen(&buf[15]);
     nickname=malloc(length+strlen("guest-")+1);
     sprintf(nickname, "guest-%s", &(buf[15]));
     return 1;
   }
+  if(!strncmp(buf, "Captcha: ", 9))
+  {
+    gtk_widget_hide(GTK_WIDGET(gtk_builder_get_object(gui, "main")));
+    gtk_widget_show_all(GTK_WIDGET(gtk_builder_get_object(gui, "captcha")));
+    char link[snprintf(0,0,"Captcha: <a href=\"%s\">%s</a>", &buf[9], &buf[9])+1];
+    sprintf(link, "Captcha: <a href=\"%s\">%s</a>", &buf[9], &buf[9]);
+    gtk_label_set_markup(GTK_LABEL(gtk_builder_get_object(gui, "captcha_link")), link);
+    return 1;
+  }
   // Start streams once we're properly connected
   if(!strncmp(buf, "Currently on cam: ", 18))
   {
@@ -317,7 +326,9 @@ gboolean handledata(GIOChannel* iochannel, GIOCondition condition, gpointer data
         {
           memmove(space, msg, strlen(msg)+1);
           char* end=strchr(nick, ':');
-          pm=strndup(nick, end-nick);
+          pm=malloc(end-nick+1);
+          memcpy(pm, nick, end-nick);
+          pm[end-nick]=0;
         }
       }
     }
@@ -523,11 +534,20 @@ gboolean handledata(GIOChannel* iochannel, GIOCondition condition, gpointer data
     g_io_channel_read_chars(iochannel, (gchar*)pkt.data+pos, size-pos, &r, 0);
     pos+=r;
   }
+  unsigned int scalewidth=data->scalewidth;
+  unsigned int scaleheight=data->scaleheight;
   if(!strcmp(&buf[7], "out"))
   {
-    dprintf(tc_client_in[1], "/video %i\n", size+1);
-    write(tc_client_in[1], &frameinfo, 1);
-    write(tc_client_in[1], pkt.data, size);
+    if(cam)
+    {
+      dprintf(tc_client_in[1], "/video %i\n", size+1);
+      write(tc_client_in[1], &frameinfo, 1);
+      write(tc_client_in[1], pkt.data, size);
+    }else{
+      cam=&campreview;
+      scalewidth=320;
+      scaleheight=240;
+    }
   }
   if((frameinfo&0xf)!=2){return 1;} // Not FLV1, get data but discard it
   if(!cam){printf("No cam found with ID '%s'\n", &buf[7]); return 1;}
@@ -537,15 +557,15 @@ gboolean handledata(GIOChannel* iochannel, GIOCondition condition, gpointer data
   if(!gotframe){return 1;}
 
   // Scale and convert to RGB24 format
-  unsigned int bufsize=avpicture_get_size(PIX_FMT_RGB24, data->scalewidth, data->scaleheight);
+  unsigned int bufsize=avpicture_get_size(PIX_FMT_RGB24, scalewidth, scaleheight);
   unsigned char buf[bufsize];
   cam->dstframe->data[0]=buf;
-  cam->dstframe->linesize[0]=data->scalewidth*3;
-  struct SwsContext* swsctx=sws_getContext(cam->frame->width, cam->frame->height, cam->frame->format, data->scalewidth, data->scaleheight, AV_PIX_FMT_RGB24, 0, 0, 0, 0);
+  cam->dstframe->linesize[0]=scalewidth*3;
+  struct SwsContext* swsctx=sws_getContext(cam->frame->width, cam->frame->height, cam->frame->format, scalewidth, scaleheight, AV_PIX_FMT_RGB24, 0, 0, 0, 0);
   sws_scale(swsctx, (const uint8_t*const*)cam->frame->data, cam->frame->linesize, 0, cam->frame->height, cam->dstframe->data, cam->dstframe->linesize);
   sws_freeContext(swsctx);
 
-  GdkPixbuf* gdkframe=gdk_pixbuf_new_from_data(cam->dstframe->data[0], GDK_COLORSPACE_RGB, 0, 8, data->scalewidth, data->scaleheight, cam->dstframe->linesize[0], 0, 0);
+  GdkPixbuf* gdkframe=gdk_pixbuf_new_from_data(cam->dstframe->data[0], GDK_COLORSPACE_RGB, 0, 8, scalewidth, scaleheight, cam->dstframe->linesize[0], 0, 0);
   gtk_image_set_from_pixbuf(GTK_IMAGE(cam->cam), gdkframe);
   // Make sure it gets redrawn in time
   gdk_window_process_updates(gtk_widget_get_window(cam->cam), 1);
@@ -575,9 +595,8 @@ void audiothread(int fd)
 }
 #endif
 
-#ifdef HAVE_V4L2
-pid_t camproc=0;
-unsigned int cameventsource;
+#ifdef HAVE_CAM
+unsigned int cameventsource=0;
 void togglecam(GtkCheckMenuItem* item, struct viddata* data)
 {
   if(!gtk_check_menu_item_get_active(item))
@@ -585,86 +604,20 @@ void togglecam(GtkCheckMenuItem* item, struct viddata* data)
     g_source_remove(cameventsource);
     kill(camproc, SIGINT);
     camproc=0;
-    dprintf(tc_client_in[1], "/camdown\n");
-    dprintf(tc_client[1], "VideoEnd: out\n"); // Close our local display
+    if(camera_find("out"))
+    {
+      dprintf(tc_client_in[1], "/camdown\n");
+      dprintf(tc_client[1], "VideoEnd: out\n"); // Close our local display
+    }
     return;
   }
-  // Set up a second pipe to be handled by handledata() to avoid overlap with tc_client's output
-  int campipe[2];
-  pipe(campipe);
-  dprintf(tc_client_in[1], "/camup\n");
-// printf("Camming up!\n");
-  camproc=fork();
-  if(!camproc)
-  {
-    close(campipe[0]);
-    prctl(PR_SET_PDEATHSIG, SIGHUP);
-    unsigned int delay=500000;
-    // Set up camera
-    int fd=v4l2_open("/dev/video0", O_RDWR);
-    struct v4l2_format fmt;
-    fmt.type=V4L2_BUF_TYPE_VIDEO_CAPTURE;
-    fmt.fmt.pix.width=320;
-    fmt.fmt.pix.height=240;
-    fmt.fmt.pix.pixelformat=V4L2_PIX_FMT_RGB24;
-    fmt.fmt.pix.field=V4L2_FIELD_NONE;
-    fmt.fmt.pix.bytesperline=fmt.fmt.pix.width*3;
-    fmt.fmt.pix.sizeimage=fmt.fmt.pix.bytesperline*fmt.fmt.pix.height;
-    v4l2_ioctl(fd, VIDIOC_S_FMT, &fmt);
-    AVCodecContext* ctx=avcodec_alloc_context3(data->vencoder);
-    ctx->width=fmt.fmt.pix.width;
-    ctx->height=fmt.fmt.pix.height;
-    ctx->pix_fmt=PIX_FMT_YUV420P;
-    ctx->time_base.num=1;
-    ctx->time_base.den=10;
-    avcodec_open2(ctx, data->vencoder, 0);
-    AVFrame* frame=av_frame_alloc();
-    frame->format=PIX_FMT_RGB24;
-    frame->width=fmt.fmt.pix.width;
-    frame->height=fmt.fmt.pix.height;
-    av_image_alloc(frame->data, frame->linesize, ctx->width, ctx->height, frame->format, 1);
-    AVPacket packet;
-    packet.buf=0;
-    packet.data=0;
-    packet.size=0;
-    packet.dts=AV_NOPTS_VALUE;
-    packet.pts=AV_NOPTS_VALUE;
+  GtkWidget* window=GTK_WIDGET(gtk_builder_get_object(gui, "camselection"));
+  gtk_widget_show_all(window);
 
-    // Set up frame for conversion from the camera's format to a format the encoder can use
-    AVFrame* dstframe=av_frame_alloc();
-    dstframe->format=ctx->pix_fmt;
-    dstframe->width=ctx->width;
-    dstframe->height=ctx->height;
-    av_image_alloc(dstframe->data, dstframe->linesize, ctx->width, ctx->height, ctx->pix_fmt, 1);
-
-    struct SwsContext* swsctx=sws_getContext(frame->width, frame->height, PIX_FMT_RGB24, frame->width, frame->height, AV_PIX_FMT_YUV420P, 0, 0, 0, 0);
-
-    while(1)
-    {
-      usleep(delay);
-      if(delay>100000){delay-=50000;}
-      v4l2_read(fd, frame->data[0], fmt.fmt.pix.sizeimage);
-      int gotpacket;
-      sws_scale(swsctx, (const uint8_t*const*)frame->data, frame->linesize, 0, frame->height, dstframe->data, dstframe->linesize);
-      av_init_packet(&packet);
-      packet.data=0;
-packet.size=0;
-      avcodec_encode_video2(ctx, &packet, dstframe, &gotpacket);
-      unsigned char frameinfo=0x22; // Note: differentiating between keyframes and non-keyframes seems to break stuff, so let's just go with all being interframes (1=keyframe, 2=interframe, 3=disposable interframe)
-      // Send the packet to our main thread so we can see ourselves (the main thread also sends it to the server)
-      dprintf(campipe[1], "Video: out %i\n", packet.size+1);
-      write(campipe[1], &frameinfo, 1);
-      write(campipe[1], packet.data, packet.size);
-
-      av_free_packet(&packet);
-    }
-    sws_freeContext(swsctx);
-    _exit(0);
-  }
-  close(campipe[1]);
-  GIOChannel* channel=g_io_channel_unix_new(campipe[0]);
-  g_io_channel_set_encoding(channel, 0, 0);
-  cameventsource=g_io_add_watch(channel, G_IO_IN, handledata, data);
+  // Start a cam thread for the selected cam
+  GtkComboBox* combo=GTK_COMBO_BOX(gtk_builder_get_object(gui, "camselect_combo"));
+  GIOChannel* channel=camthread(gtk_combo_box_get_active_id(combo), data->vencoder, 100000);
+  cameventsource=g_io_add_watch(channel, G_IO_IN, handledata, 0);
 }
 #endif
 
@@ -798,7 +751,9 @@ void sendmessage(GtkEntry* entry, struct viddata* data)
     const char* end=strchr(&msg[5], ' ');
     if(end)
     {
-      pm=strndup(&msg[5], end-&msg[5]);
+      pm=malloc(end-&msg[5]+1);
+      memcpy(pm, &msg[5], end-&msg[5]);
+      pm[end-&msg[5]]=0;
       struct user* user=finduser(pm);
       if(!user)
       {
@@ -906,12 +861,18 @@ void startsession(GtkButton* button, void* x)
   }
 #endif
   if(acc_user[0]){dprintf(tc_client_in[1], "%s\n", acc_pass);}
-  write(tc_client_in[1], "/color\n", 7);
   GIOChannel* tcchannel=g_io_channel_unix_new(tc_client[0]);
   g_io_channel_set_encoding(tcchannel, 0, 0);
   g_io_add_watch(tcchannel, G_IO_IN, handledata, data);
 }
 
+void captcha_done(GtkWidget* button, void* x)
+{
+  gtk_widget_hide(GTK_WIDGET(gtk_builder_get_object(gui, "captcha")));
+  gtk_widget_show_all(GTK_WIDGET(gtk_builder_get_object(gui, "main")));
+  write(tc_client_in[1], "\n", 1);
+}
+
 int main(int argc, char** argv)
 {
   if(!strncmp(argv[0], "./", 2)){frombuild=1;}
@@ -963,10 +924,40 @@ int main(int argc, char** argv)
   }
   gtk_builder_connect_signals(gui, 0);
 
-#ifdef HAVE_V4L2
+  unsigned int i;
+#ifdef HAVE_CAM
   GtkWidget* item=GTK_WIDGET(gtk_builder_get_object(gui, "menuitem_broadcast_camera"));
   g_signal_connect(item, "toggled", G_CALLBACK(togglecam), data);
   data->vencoder=avcodec_find_encoder(AV_CODEC_ID_FLV1);
+  // Set up cam selection and preview
+  campreview.cam=GTK_WIDGET(gtk_builder_get_object(gui, "camselect_preview"));
+  campreview.frame=av_frame_alloc();
+  campreview.dstframe=av_frame_alloc();
+  campreview.vctx=avcodec_alloc_context3(data->vdecoder);
+  avcodec_open2(campreview.vctx, data->vdecoder, 0);
+  GtkComboBox* combo=GTK_COMBO_BOX(gtk_builder_get_object(gui, "camselect_combo"));
+  g_signal_connect(combo, "changed", G_CALLBACK(camselect_change), data->vencoder);
+  // Signals for cancelling
+  item=GTK_WIDGET(gtk_builder_get_object(gui, "camselection"));
+  g_signal_connect(item, "delete-event", G_CALLBACK(camselect_cancel), 0);
+  item=GTK_WIDGET(gtk_builder_get_object(gui, "camselect_cancel"));
+  g_signal_connect(item, "clicked", G_CALLBACK(camselect_cancel), 0);
+  // Signals for switching from preview to streaming
+  item=GTK_WIDGET(gtk_builder_get_object(gui, "camselect_ok"));
+  g_signal_connect(item, "clicked", G_CALLBACK(camselect_accept), data->vencoder);
+  // Populate list of cams
+  unsigned int count;
+  char** cams=cam_list(&count);
+  GtkListStore* list=gtk_list_store_new(1, G_TYPE_STRING);
+  for(i=0; i<count; ++i)
+  {
+    gtk_list_store_insert_with_values(list, 0, -1, 0, cams[i], -1);
+    free(cams[i]);
+  }
+  free(cams);
+  gtk_combo_box_set_model(combo, GTK_TREE_MODEL(list));
+  g_object_unref(list);
+  gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(combo), gtk_cell_renderer_text_new(), 1);
 #else
   GtkWidget* item=GTK_WIDGET(gtk_builder_get_object(gui, "menuitem_broadcast"));
   gtk_widget_destroy(item);
@@ -985,6 +976,9 @@ int main(int argc, char** argv)
 
   GtkWidget* panes=GTK_WIDGET(gtk_builder_get_object(gui, "vpaned"));
   g_signal_connect(panes, "notify::position", G_CALLBACK(handleresizepane), data);
+  gtk_paned_set_wide_handle(GTK_PANED(panes), 1);
+  panes=GTK_WIDGET(gtk_builder_get_object(gui, "chatnick"));
+  gtk_paned_set_wide_handle(GTK_PANED(panes), 1);
 
   GtkWidget* inputfield=GTK_WIDGET(gtk_builder_get_object(gui, "inputfield"));
   g_signal_connect(inputfield, "activate", G_CALLBACK(sendmessage), data);
@@ -1023,6 +1017,8 @@ int main(int argc, char** argv)
   // Connect signal for tab changing (to un-highlight)
   item=GTK_WIDGET(gtk_builder_get_object(gui, "tabs"));
   g_signal_connect(item, "switch-page", G_CALLBACK(pm_select), 0);
+  // Connect signal for captcha
+  g_signal_connect(gtk_builder_get_object(gui, "captcha_done"), "clicked", G_CALLBACK(captcha_done), 0);
   // Populate saved channels
   GtkWidget* startbox=GTK_WIDGET(gtk_builder_get_object(gui, "startbox"));
   int channelcount=config_get_int("channelcount");
@@ -1031,7 +1027,6 @@ int main(int argc, char** argv)
     gtk_widget_destroy(GTK_WIDGET(gtk_builder_get_object(gui, "channel_placeholder")));
   }
   char buf[256];
-  int i;
   for(i=0; i<channelcount; ++i)
   {
     GtkWidget* box=gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
