@@ -19,7 +19,7 @@
 extern SECURITY_ATTRIBUTES sa;
 #define w32_runcmd(cmd) \
   { \
-    char* arg=strchr(cmd,''); \
+    char* arg=strchr(cmd,' '); \
     if(arg){arg[0]=0; arg=&arg[1];} \
     ShellExecute(0, "open", cmd, arg, 0, SW_SHOWNORMAL); \
   }
@@ -30,8 +30,12 @@ extern SECURITY_ATTRIBUTES sa;
   extern GtkWidget* gtk_box_new(int vertical, int spacing);
   extern int gtk_widget_get_allocated_width(GtkWidget* widget);
   extern int gtk_widget_get_allocated_height(GtkWidget* widget);
+  extern const char* gtk_combo_box_get_active_id(GtkComboBox* combo);
 #endif
 #if GTK_MAJOR_VERSION<3 || (GTK_MAJOR_VERSION==3 && GTK_MINOR_VERSION<10)
   #define gtk_button_new_from_icon_name(name, size) gtk_button_new_from_stock(name)
   extern GtkBuilder* gtk_builder_new_from_file(const char* filename);
 #endif
+#if GTK_MAJOR_VERSION<3 || (GTK_MAJOR_VERSION==3 && GTK_MINOR_VERSION<16)
+extern void gtk_paned_set_wide_handle(void*, char);
+#endif
