@@ -28,8 +28,10 @@ struct camera
   GtkWidget* box; // holds label and cam
   GtkWidget* label;
 };
+extern struct camera campreview;
 extern struct camera* cams;
 extern unsigned int camcount;
+extern pid_t camproc;
 
 #if defined(HAVE_AVRESAMPLE) || defined(HAVE_SWRESAMPLE)
 extern void camera_playsnd(int audiopipe, struct camera* cam, short* samples, unsigned int samplecount);
@@ -40,3 +42,9 @@ extern struct camera* camera_find(const char* id);
 extern struct camera* camera_findbynick(const char* nick);
 extern struct camera* camera_new(void);
 extern void camera_cleanup(void);
+#ifdef HAVE_CAM
+extern GIOChannel* camthread(const char* name, AVCodec* vencoder, unsigned int delay);
+extern void camselect_change(GtkComboBox* combo, AVCodec* vencoder);
+extern gboolean camselect_cancel(GtkWidget* widget, void* x1, void* x2);
+extern void camselect_accept(GtkWidget* widget, AVCodec* vencoder);
+#endif
